from http import HTTPStatus
from typing import Any, Dict, Optional, Union, cast

import httpx

from ... import errors
from ...client import Client
from ...models.control_logic import ControlLogic
from ...models.http_validation_error import HTTPValidationError
from ...models.post_item_resource_item_id_control_logic_post_response_post_item_resource_item_id_controllogic_post import (
    PostItemResourceItemIdControlLogicPostResponsePostItemResourceItemIdControllogicPost,
)
from ...types import Response


def _get_kwargs(
    item_id: str,
    *,
    client: Client,
    json_body: ControlLogic,
) -> Dict[str, Any]:
    url = "{}/Resource/{item_id}/ControlLogic/".format(client.base_url, item_id=item_id)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    json_json_body = json_body.to_dict()

    return {
        "method": "post",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "follow_redirects": client.follow_redirects,
        "json": json_json_body,
    }


def _parse_response(
    *, client: Client, response: httpx.Response
) -> Optional[
    Union[
        Any, HTTPValidationError, PostItemResourceItemIdControlLogicPostResponsePostItemResourceItemIdControllogicPost
    ]
]:
    if response.status_code == HTTPStatus.OK:
        response_200 = PostItemResourceItemIdControlLogicPostResponsePostItemResourceItemIdControllogicPost.from_dict(
            response.json()
        )

        return response_200
    if response.status_code == HTTPStatus.NOT_FOUND:
        response_404 = cast(Any, None)
        return response_404
    if response.status_code == HTTPStatus.UNPROCESSABLE_ENTITY:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Client, response: httpx.Response
) -> Response[
    Union[
        Any, HTTPValidationError, PostItemResourceItemIdControlLogicPostResponsePostItemResourceItemIdControllogicPost
    ]
]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    item_id: str,
    *,
    client: Client,
    json_body: ControlLogic,
) -> Response[
    Union[
        Any, HTTPValidationError, PostItemResourceItemIdControlLogicPostResponsePostItemResourceItemIdControllogicPost
    ]
]:
    """Post Item

    Args:
        item_id (str):
        json_body (ControlLogic): Submodel to describe the control logic of a resource, by
            describing its control policy. It specifies in which sequence the resource processes the
            products.

            Args:
                id (str): The id of the control logic.
                id_short (str): The short id of the control logic.
                description (Optional[str]): The description of the control logic.
                semantic_id (Optional[str]): The semantic id of the control logic.
                sequencing_policy (Literal["FIFO", "SPT_transport", "LIFO", "SPT", "EDD", "ODD"]): The
            sequencing policy of the resource, determining in which sequence requests are processed.
                routing_policy (Literal["random", "nearest", "shortest_queue", "alternating,
            "round_robin"]): The routing policy of the resource how redundant sub resources are used.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, HTTPValidationError, PostItemResourceItemIdControlLogicPostResponsePostItemResourceItemIdControllogicPost]]
    """

    kwargs = _get_kwargs(
        item_id=item_id,
        client=client,
        json_body=json_body,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    item_id: str,
    *,
    client: Client,
    json_body: ControlLogic,
) -> Optional[
    Union[
        Any, HTTPValidationError, PostItemResourceItemIdControlLogicPostResponsePostItemResourceItemIdControllogicPost
    ]
]:
    """Post Item

    Args:
        item_id (str):
        json_body (ControlLogic): Submodel to describe the control logic of a resource, by
            describing its control policy. It specifies in which sequence the resource processes the
            products.

            Args:
                id (str): The id of the control logic.
                id_short (str): The short id of the control logic.
                description (Optional[str]): The description of the control logic.
                semantic_id (Optional[str]): The semantic id of the control logic.
                sequencing_policy (Literal["FIFO", "SPT_transport", "LIFO", "SPT", "EDD", "ODD"]): The
            sequencing policy of the resource, determining in which sequence requests are processed.
                routing_policy (Literal["random", "nearest", "shortest_queue", "alternating,
            "round_robin"]): The routing policy of the resource how redundant sub resources are used.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, HTTPValidationError, PostItemResourceItemIdControlLogicPostResponsePostItemResourceItemIdControllogicPost]
    """

    return sync_detailed(
        item_id=item_id,
        client=client,
        json_body=json_body,
    ).parsed


async def asyncio_detailed(
    item_id: str,
    *,
    client: Client,
    json_body: ControlLogic,
) -> Response[
    Union[
        Any, HTTPValidationError, PostItemResourceItemIdControlLogicPostResponsePostItemResourceItemIdControllogicPost
    ]
]:
    """Post Item

    Args:
        item_id (str):
        json_body (ControlLogic): Submodel to describe the control logic of a resource, by
            describing its control policy. It specifies in which sequence the resource processes the
            products.

            Args:
                id (str): The id of the control logic.
                id_short (str): The short id of the control logic.
                description (Optional[str]): The description of the control logic.
                semantic_id (Optional[str]): The semantic id of the control logic.
                sequencing_policy (Literal["FIFO", "SPT_transport", "LIFO", "SPT", "EDD", "ODD"]): The
            sequencing policy of the resource, determining in which sequence requests are processed.
                routing_policy (Literal["random", "nearest", "shortest_queue", "alternating,
            "round_robin"]): The routing policy of the resource how redundant sub resources are used.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, HTTPValidationError, PostItemResourceItemIdControlLogicPostResponsePostItemResourceItemIdControllogicPost]]
    """

    kwargs = _get_kwargs(
        item_id=item_id,
        client=client,
        json_body=json_body,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    item_id: str,
    *,
    client: Client,
    json_body: ControlLogic,
) -> Optional[
    Union[
        Any, HTTPValidationError, PostItemResourceItemIdControlLogicPostResponsePostItemResourceItemIdControllogicPost
    ]
]:
    """Post Item

    Args:
        item_id (str):
        json_body (ControlLogic): Submodel to describe the control logic of a resource, by
            describing its control policy. It specifies in which sequence the resource processes the
            products.

            Args:
                id (str): The id of the control logic.
                id_short (str): The short id of the control logic.
                description (Optional[str]): The description of the control logic.
                semantic_id (Optional[str]): The semantic id of the control logic.
                sequencing_policy (Literal["FIFO", "SPT_transport", "LIFO", "SPT", "EDD", "ODD"]): The
            sequencing policy of the resource, determining in which sequence requests are processed.
                routing_policy (Literal["random", "nearest", "shortest_queue", "alternating,
            "round_robin"]): The routing policy of the resource how redundant sub resources are used.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, HTTPValidationError, PostItemResourceItemIdControlLogicPostResponsePostItemResourceItemIdControllogicPost]
    """

    return (
        await asyncio_detailed(
            item_id=item_id,
            client=client,
            json_body=json_body,
        )
    ).parsed
